<?php

if(!defined('IN_PRGM')) return false;

if(!class_exists('ArticlesClass'))
{
class ArticlesClass
{
  public  $article          = false;
  public  $pluginid         = 0;
  public  $p_prefix         = '';
  public  $globalsettings   = 0;
  public  $language         = array();
  public  $settings         = array();
  public  $slug_arr         = array(); //SD343
  public  $table_name       = '';
  public  $authormode       = false;

  //SD351: attachments related
  public  $attach_path        = '';
  public  $attach_path_ok     = false;
  public  $attach_path_exists = false;
  public  $attach_extentions  = '';
  public  $attach_max_size    = 0;
  public  $doSearch           = false; //SD350
  public  $SearchTerm         = false; //SD350

  private $_attachment        = null;
  private $_IsAdmin           = false;
  private $_IsSiteAdmin       = false;
  private $_HasAdminRights    = false;
  private $_candownload       = false;

  private $_plugin_folder   = '';
  private $_quoted_url_ext  = '';
  private $_smarty          = false;
  private $_rating          = false; //SD342
  private $_tag_active      = false; //SD343
  private $_isTagPage       = false; //SD343
  private $_articleCounter  = 0; //SD344

  // Article Bitfield Settings (static, so use "self::$articlebitfield"!)
  public static $articlebitfield = array( // max. 32 options!
    'useglobalsettings'       => 1,
    'displayonline'           => 2,
    'displaytitle'            => 4,
    'displayauthor'           => 8,
    'displaycreateddate'      => 16,
    'displayupdateddate'      => 32,
    'displayprintarticlelink' => 64,
    'displayemailarticlelink' => 128,
    'displaydescription'      => 256,
    'displaysmilies'          => 512,
    'displaycomments'         => 1024,
    'displayviews'            => 2048,
    'displaymainpage'         => 4096,
    'displaysticky'           => 8192,
    'displayratings'          => 16384,
    'displaysocialmedia'      => 32768,
    'displaytags'             => 65536,
    'displaypdfarticlelink'   => 131072,
    'displayaspopup'          => 262144,
    'ignoreexcerptmode'       => 524288, //SD342
    'linktomainpage'          => 1048576, //SD343
    );

  public function __construct($plugin_folder)
  {
    global $DB, $mainsettings_url_extension, $userinfo;

    $this->_plugin_folder = $plugin_folder;
    $this->pluginid = GetPluginIDbyFolder($this->_plugin_folder);
    $this->_quoted_url_ext = SD_QUOTED_URL_EXT;

    // Get plugin's language phrases and settings
    $this->language = GetLanguage($this->pluginid);
    $this->settings = GetPluginSettings($this->pluginid);
    $this->p_prefix = 'p'.$this->pluginid.'_';
    $this->table_name = $this->p_prefix . 'news';

    //SD351: check attachments path
    $this->attach_path = '';
    $this->attach_path_ok = $this->attach_path_exists = false;
    if(isset($this->settings['attachments_upload_path']) &&
       strlen(trim($this->settings['attachments_upload_path'])))
    {
      $this->attach_path = trim($this->settings['attachments_upload_path']);
      $this->attach_path_exists = @is_dir(ROOT_PATH.$this->attach_path);
      $this->attach_path_ok = $this->attach_path_exists &&
                              @is_writable(ROOT_PATH.$this->attach_path);
    }
    $this->attach_extentions = (empty($this->settings['valid_attachment_types'])?'':
                               (string)$this->settings['valid_attachment_types']);
    $this->attach_max_size   = (empty($this->settings['attachments_max_size'])?0:
                               (int)$this->settings['attachments_max_size']);

    $this->authormode        = empty($userinfo['adminaccess']) &&
                               !empty($userinfo['authormode']);

    $this->_candownload    = (!empty($userinfo['plugindownloadids']) &&
                             @in_array($this->pluginid, $userinfo['plugindownloadids']));
    $this->_IsAdmin        = (!empty($userinfo['pluginadminids']) &&
                              @in_array($this->pluginid, $userinfo['pluginadminids']));
    $this->_IsSiteAdmin    = !empty($userinfo['adminaccess']) && !empty($userinfo['loggedin']) &&
                             !empty($userinfo['userid']);
    $this->_HasAdminRights = $this->_IsSiteAdmin ||
                             (!$this->authormode &&
                              ((!empty($userinfo['admin_pages']) && in_array('articles', $userinfo['admin_pages'])) &&
                               (!empty($userinfo['pluginadminids']) && in_array($this->pluginid, $userinfo['pluginadminids']))));

    // create global article settings based on plugin settings
    $this->globalsettings = 0;
    $this->globalsettings += $this->settings['display_title']                  == 1 ?      4 : 0;
    $this->globalsettings += $this->settings['display_author']                 == 1 ?      8 : 0;
    $this->globalsettings += $this->settings['display_creation_date']          == 1 ?     16 : 0;
    $this->globalsettings += $this->settings['display_updated_date']           == 1 ?     32 : 0;
    $this->globalsettings += $this->settings['display_print_link']             == 1 ?     64 : 0;
    $this->globalsettings += $this->settings['display_email_link']             == 1 ?    128 : 0;
    $this->globalsettings += $this->settings['display_description_in_article'] == 1 ?    256 : 0;
    $this->globalsettings += $this->settings['display_comments']               == 1 ?   1024 : 0;
    $this->globalsettings += $this->settings['display_views_count']            == 1 ?   2048 : 0;
    $this->globalsettings += $this->settings['display_on_main_page']           == 1 ?   4096 : 0;
    $this->globalsettings += $this->settings['sticky_article']                 == 1 ?   8192 : 0;
    $this->globalsettings += $this->settings['display_user_ratings']           == 1 ?  16384 : 0;
    $this->globalsettings += $this->settings['display_social_media_links']     == 1 ?  32768 : 0;
    $this->globalsettings += !empty($this->settings['display_tags'])                ?  65536 : 0;
    $this->globalsettings += !empty($this->settings['display_pdf_link'])            ? 131072 : 0;
    $this->globalsettings += !empty($this->settings['display_as_popup'])            ? 262144 : 0;
    $this->globalsettings += !empty($this->settings['ignore_excerpt_mode'])         ? 524288 : 0; //SD342
    $this->globalsettings += !empty($this->settings['link_to_main_page'])           ?1048576 : 0; //SD343

    require_once(SD_INCLUDE_PATH.'class_sd_tags.php');

  } //ArticlesClass


  // ##########################################################################
  // INIT ATTACHMENT
  // ##########################################################################

  function InitAttachment()
  {
    global $DB, $sdlanguage, $userinfo;

    require_once(SD_INCLUDE_PATH.'class_sd_attachment.php');

    if(!empty($this->_attachment) && ($this->_attachment instanceof SD_Attachment))
    {
      return true;
    }

    $this->_attachment = new SD_Attachment($this->pluginid,'articles');
    $this->_attachment->setValidExtensions($this->attach_extentions);
    $this->_attachment->setMaxFilesizeKB($this->attach_max_size);
    if($this->attach_path_ok)
    {
      $this->_attachment->setStorageBasePath($this->attach_path);
    }

    return true;

  } //InitAttachment


  private function InitTemplate()
  {
    global $categoryid, $mainsettings, $sdlanguage, $userinfo;

    // Instantiate smarty object
    //$this->_smarty = SD_Smarty::getNew(); //SD342
    $this->_smarty = SD_Smarty::getInstance(); //SD343
    $this->_smarty->assign('AdminAccess',   ($userinfo['adminaccess'] || (!empty($userinfo['pluginadminids']) &&
                                            @array_key_exists($this->pluginid,array_flip($userinfo['pluginadminids'])))));
    $this->_smarty->assign('categoryid',    $categoryid);
    $this->_smarty->assign('pluginid',      $this->pluginid);
    $this->_smarty->assign('sdlanguage',    $sdlanguage);
    $this->_smarty->assign('sdurl',         SITE_URL);
    $this->_smarty->assign('securitytoken', $userinfo['securitytoken']);
    $this->_smarty->assign('article_lang',  $this->language);
    $this->_smarty->assign('page_title',    (defined('SD_CATEGORY_TITLE')?SD_CATEGORY_TITLE:''));
    $this->_smarty->assign('enable_rss',    !empty($mainsettings['enable_rss']));
    $this->_smarty->error_reporting = E_ALL & ~E_NOTICE;

    $this->_smarty->assign('article_rating_form', '');
    $this->_smarty->assign('article_display_tags', false);
    $this->_smarty->assign('article_tags', array());
    $this->_smarty->assign('article_tagcount', 0);
    $this->_smarty->assign('article_display_article', false);
    $this->_smarty->assign('article_display_description', false);
    $this->_smarty->assign('article_display_pagination', false);
    $this->_smarty->assign('article_description', '');
    $this->_smarty->assign('article_pagination', '');
    $this->_smarty->assign('article_read_more', false);
    $this->_smarty->assign('article_published', '');
    $this->_smarty->assign('article_updated', '');
    $this->_smarty->assign('article_display_author', false);
    $this->_smarty->assign('article_display_published', false);
    $this->_smarty->assign('article_display_updated', false);
    $this->_smarty->assign('article_display_footer', false);
    $this->_smarty->assign('article_link', '');
    $this->_smarty->assign('article_title_link', true);
    $this->_smarty->assign('article_share_title', '');
    $this->_smarty->assign('article_share_url', '');
    $this->_smarty->assign('article_display_social', false);
    $this->_smarty->assign('article_display_email', false);
    $this->_smarty->assign('article_display_print', false);
    $this->_smarty->assign('article_display_pdf', false);
    $this->_smarty->assign('article_display_comments', false);

  } //InitTemplate


  public function GetArticle($article_id)
  {
    global $DB;
    $DB->result_type = MYSQL_ASSOC;
    $this->article = $DB->query_first('SELECT * FROM {'.$this->table_name.'} WHERE articleid = %d', $article_id);
  } //GetArticle


  // ##########################################################################
  // DISPLAY ARTICLE
  // ##########################################################################

  public function DisplayArticle($article_arr, $viewing_all_articles = false, $preselected=false)
  {
    global $sdlanguage, $userinfo;

    //SD342 check for article being active or not AND
    //      if current user has view permissions for it
    $article_groups = isset($article_arr['access_view'])?sd_ConvertStrToArray($article_arr['access_view'],'|'):array();
    //SD343: check all usergroups, not only primary
    $hasAccess = !empty($userinfo['adminaccess']) ||
                 empty($article_groups) ||
                 (is_array($article_groups) && count(array_intersect($userinfo['usergroupids'],$article_groups)));
    if(!$hasAccess || empty($article_arr['settings']) || (($article_arr['settings'] & 2) == 0))
    {
      DisplayMessage($sdlanguage['no_view_access'], true);
      return false;
    }

    global $DB, $Comments, $categoryid, $categoryids, $mainsettings, $mainsettings_url_extension,
           $mainsettings_modrewrite, $sdurl, $usersystem;

    // check which page of the article we are on
    // this variable needs to be created before displaying the article becuase it's used
    // in the description printing area
    $currentpage = GetVar($this->p_prefix.'page', 1, 'whole_number');

    // use global settings for this article?
    if($article_arr['settings'] & self::$articlebitfield['useglobalsettings'])
    {
      $article_arr['settings'] = $this->globalsettings;
    }

    // *** SMARTY TEMPLATE ***
    $this->InitTemplate();
    $article_display_title = $article_arr['settings'] & self::$articlebitfield['displaytitle'];
    $this->_smarty->assign('plugin_id', $this->pluginid); //SD342
    $this->_smarty->assign('plugin_folder', $this->_plugin_folder); //SD342
    $this->_smarty->assign('article_id', $article_arr['articleid']);
    $this->_smarty->assign('article_title_popup', false);

    //SD351: article link generation moved to GetArticleLink (functions_global.php)
    $article_link = GetArticleLink($categoryid, $this->pluginid, $article_arr,
                                   self::$articlebitfield,
                                   $this->_isTagPage, $this->_plugin_folder); //SD351

    $this->_smarty->assign('article_display_title', $article_display_title);
    $this->_smarty->assign('article_title_link', $viewing_all_articles && $this->settings['title_link']);
    $this->_smarty->assign('article_title', $article_arr['title']);
    $this->_smarty->assign('article_link', $article_link);
    //SD342
    $this->_smarty->assign('article_thumbnail', $article_arr['thumbnail']);
    $this->_smarty->assign('article_featured', !empty($article_arr['featured']));
    $this->_smarty->assign('article_featuredpath', $article_arr['featuredpath']);
    $this->_smarty->assign('article_rating', $article_arr['rating']);

    if($article_display_title)
    {
      // SD313: 2010-10-16 - Display rating html (next line)
      if(function_exists('GetRatingForm') && ($article_arr['settings'] & self::$articlebitfield['displayratings']))
      {
        $rating_form = GetRatingForm('p'.$this->pluginid.'-'.$article_arr['articleid'], $this->pluginid);
        $this->_smarty->assign('article_rating_form', $rating_form);
        //SD342
        $rating = new rating('p'.$this->pluginid.'-'.$article_arr['articleid'], $this->pluginid);
        if($rating->votes)
        {
          $this->_smarty->assign('article_user_rating', ((double)$rating->average/20));
        }
        unset($rating);
      }
    }

    // ######################## Article Tags ####################################
    // SD341 - TAGS display
    if($article_arr['settings'] & self::$articlebitfield['displaytags'])
    {
      $article_arr['tags'] = false;
      //SD343: fetch tags from global table
      $tags = SD_Tags::GetPluginTags($this->pluginid,$article_arr['articleid']);
      if(!empty($tags)) $article_arr['tags'] = implode(',', $tags);
      unset($tags);

      if(!empty($article_arr['tags']))
      {
        global $mainsettings_tag_results_page;
        $this->_smarty->assign('article_display_tags', true);
        $article_arr['tags'] = explode(',',$article_arr['tags']);
        foreach($article_arr['tags'] as $tag)
        {
          $tag = rtrim(ltrim($tag));
          if(strlen($tag))
          {
            $tag_link = RewriteLink('index.php?categoryid='.$mainsettings_tag_results_page);
            $tags_arr[$tag] = preg_replace('#'.$this->_quoted_url_ext.'$#','/'.urlencode($tag), $tag_link);
          }
        } //foreach
        $this->_smarty->assign('article_tags', $tags_arr);
        $this->_smarty->assign('article_tagcount', count($tags_arr));
      }
    }

    // ######################## Article Description #############################
    $is_excerpt_mode_active = false; //SD343
    if(strlen($article_arr['description']))
    {
      if($viewing_all_articles || ( ($article_arr['settings'] & self::$articlebitfield['displaydescription']) && ($currentpage == 1) ) )
      {
        $this->_smarty->assign('article_display_description', true);
        //SD342 "ignoreexcerptmode"
        if(!($article_arr['settings'] & self::$articlebitfield['ignoreexcerptmode']) &&
           ($output = CheckExcerptMode($article_arr['description'], $this->settings['excerpt_mode_length'])))
        {
          $is_excerpt_mode_active = true; //SD343
          $this->_smarty->assign('article_description', $output['content'].
            '<div class="excerpt_message">'.$output['message'].'</div>');
        }
        else
        {
          $this->_smarty->assign('article_description', $article_arr['description']);
        }
      }
    }

    // ############################### Article ##################################
    if(!$viewing_all_articles || !strlen($article_arr['description']))
    {
      if(strlen($article_arr['article']))
      {
        $this->_smarty->assign('article_display_article', true);

        // find out how many pages are in this article
        $articlepages = preg_split('/{(pagebreak)\s*(.*?)}/i', $article_arr['article']);

        // a few more checks...
        if(empty($currentpage) || ($currentpage > count($articlepages)))
        {
          $currentpage = 1;
        }
        // this preg_replace code was created by shoz: phpfreaks.com/forums/index.php?showuser=14845
        $tmp = preg_replace('#(\A[\s]*<br[^>]*>[\s]*|'                  // remove <br/> at beginning of the string
        .'<br[^>]*>\s*(?=(?:\{pagebreak\}))|'                           // remove <br/> before {pagebreak}
        .'(?<=(?:\{pagebreak\}))\s*<br[^>]*>|'                          // remove <br/> after {pagebreak}
        .'<br[^>]*>[\s]*\Z)#is', '', $articlepages[$currentpage - 1]);  // remove <br/> at end of string

        if(!($article_arr['settings'] & self::$articlebitfield['ignoreexcerptmode']) &&
           ($output = CheckExcerptMode($tmp, $this->settings['excerpt_mode_length'])))
        {
          $is_excerpt_mode_active = true; //SD343
          $this->_smarty->assign('article_article', $output['content'].
            '<div class="excerpt_message">'.$output['message'].'</div>');
        }
        else
        {
          $this->_smarty->assign('article_article', $tmp);
        }

        if(count($articlepages) > 1)
        {
          $p = new pagination;
          $p->enablePrevNext = true; //SD343
          $p->items(count($articlepages));
          $p->limit(1);
          $p->parameterName($this->p_prefix.'page');
          $p->target($article_link);
          $p->currentPage($currentpage);
          $p->adjacents(3);
          $pagination = trim($p->getOutput());
          $this->_smarty->assign('article_display_pagination', (strlen(strip_tags($pagination)) > 0));
          $this->_smarty->assign('article_pagination', $pagination);
        }
      }

      // Update the views but only on the first page
      // SD313: and only if the article is fully displayed (not in list)
      if(!$viewing_all_articles && ($currentpage == 1) && !SD_IS_BOT &&
         ($userinfo['userid'] != $article_arr['org_author_id']) )
      {
        $DB->query('UPDATE {'.$this->table_name.'} SET views = IFNULL(views,0) + 1'.
                   ' WHERE articleid = %d',
                   $article_arr['articleid']);
      }
    }
    else if(strlen(trim($article_arr['article'])))
    {
      $this->_smarty->assign('article_read_more', true);
    }

    if($displayviews = $article_arr['settings'] & self::$articlebitfield['displayviews'])
    {
      // Must re-query view count since array contains a cached value
      $DB->result_type = MYSQL_ASSOC;
      if($tmp = $DB->query_first('SELECT views FROM {'.$this->table_name.'} WHERE articleid = '.$article_arr['articleid']))
      {
        $article_arr['views'] = $tmp['views'];
      }
      unset($tmp);
    }
    $this->_smarty->assign('article_display_views', $displayviews);
    $this->_smarty->assign('article_views', $article_arr['views']);
    $this->_smarty->assign('article_comment_count', 0);
    $this->_smarty->assign('article_counter', $this->_articleCounter); //SD344

    if( ($article_arr['settings'] & self::$articlebitfield['displaycreateddate']) ||
        ($article_arr['settings'] & self::$articlebitfield['displayauthor']) ||
        ( ($article_arr['settings'] & self::$articlebitfield['displayupdateddate']) && empty($article_arr['dateupdated']) ) )
    {

      // display published date?
      if($article_arr['settings'] & self::$articlebitfield['displaycreateddate'])
      {
        $this->_smarty->assign('article_display_published', true);
        $this->_smarty->assign('article_published_date', (!empty($article_arr['datestart']) ? DisplayDate($article_arr['datestart']) : DisplayDate($article_arr['datecreated'])));
      }

      //SD343: use user cache
      $tmp = SDUserCache::CacheUser($article_arr['org_author_id'], $article_arr['org_author_name'], false, true);
      $this->_smarty->assign('avatar', $tmp['avatar']);

      // display author?
      if($article_arr['settings'] & self::$articlebitfield['displayauthor'])
      {
        $this->_smarty->assign('article_display_author', true);
        if($article_arr['org_system_id']==$usersystem['usersystemid']) //SD342 linking
        {
          //SD343: if either no "User Profile" plugin is available -OR-
          // the plain author name of the article is different than the one
          // connected to the userid, than return "author":
          if(!defined('CP_PATH') || !strlen(CP_PATH) || ($tmp['username'] != $article_arr['author']))
          {
            $this->_smarty->assign('article_author', $article_arr['author']);
          }
          else
          {
            $this->_smarty->assign('article_author', $tmp['profile_link']);
          }
          $this->_smarty->assign('article_creator', $tmp['profile_link']); //SD343 new
        }
        else
        {
          $this->_smarty->assign('article_author', $article_arr['author']);
        }
      }

      // display updated date?
      if( ($article_arr['settings'] & self::$articlebitfield['displayupdateddate']) AND ($article_arr['dateupdated'] != 0) )
      {
        $this->_smarty->assign('article_display_updated', true);
        $this->_smarty->assign('article_updated_date', DisplayDate($article_arr['dateupdated']));
      }

    }

    //SD351: allow attachment to article
    $this->_smarty->assign('attachments_html', '');
    $this->_smarty->assign('attachments_list', '');
    if(!$viewing_all_articles && ($this->_HasAdminRights || $this->_candownload))
    {
      if($this->InitAttachment())
      {
        $this->_attachment->setUserid($userinfo['userid']);
        $this->_attachment->setUsername($userinfo['username']);
        $this->_attachment->setObjectID($article_arr['articleid']);

        $attachments_html = $this->_attachment->getAttachmentsListHTML(false,true);
        if(!empty($attachments_html))
        {
          $this->_smarty->assign('attachments_html', $attachments_html);
        }

        if(false !== ($attachment_arr = $this->_attachment->getAttachmentsArray()))
        {
          $this->_smarty->assign('attachments_list', $attachment_arr);
        }
        unset($attachment_arr);
      }
    }

    if( ($article_arr['settings'] & self::$articlebitfield['displayprintarticlelink']) ||
        ($article_arr['settings'] & self::$articlebitfield['displaypdfarticlelink']) ||
        ($article_arr['settings'] & self::$articlebitfield['displayemailarticlelink']) ||
        ($article_arr['settings'] & self::$articlebitfield['displaysocialmedia']) ||
        ($viewing_all_articles && ($article_arr['settings'] & self::$articlebitfield['displaycomments'])) )
    {
      $share_url = urlencode($article_link);
      $share_title = urlencode($article_arr['title']);
      $this->_smarty->assign('article_display_footer', true);
      $this->_smarty->assign('article_share_title', $share_title);
      $this->_smarty->assign('article_share_url', $share_url);

      //SD322 - display social bookmark tool buttons
      if($article_arr['settings'] & self::$articlebitfield['displaysocialmedia'])
      {
        $this->_smarty->assign('article_display_social', true);
        $this->_smarty->assign('article_twitter_text', htmlspecialchars($this->language['social_link_twitter'],ENT_COMPAT));
        $this->_smarty->assign('article_delicious_text', htmlspecialchars($this->language['social_link_delicious'],ENT_COMPAT));
        $this->_smarty->assign('article_facebook_text', htmlspecialchars($this->language['social_link_facebook'],ENT_COMPAT));
        $this->_smarty->assign('article_digg_text', htmlspecialchars($this->language['social_link_digg'],ENT_COMPAT));
      }

      //SD322: display email article link?
      if($article_arr['settings'] & self::$articlebitfield['displayemailarticlelink'])
      {
        $email_subject = str_replace('+','%20',$share_title);
        $this->_smarty->assign('article_display_email', true);
        $this->_smarty->assign('article_email_subject', $email_subject);
        $this->_smarty->assign('article_email_text', htmlspecialchars($this->language['email_to_friend'],ENT_COMPAT));
      }

      // display pdf / print article links?
      //SD322: added PDF link for logged-in users (requires HTML2PDF in includes folder!)
      if(!empty($userinfo['loggedin']) && ($article_arr['settings'] & self::$articlebitfield['displaypdfarticlelink']) &&
         file_exists(SD_INCLUDE_PATH.'/html2pdf/html2pdf.class.php'))
      {
        $this->_smarty->assign('article_display_pdf', true);
        $this->_smarty->assign('article_pdf_text', htmlspecialchars($this->language['pdf'],ENT_COMPAT));
      }
      if($article_arr['settings'] & self::$articlebitfield['displayprintarticlelink'])
      {
        $this->_smarty->assign('article_display_print', true);
        $this->_smarty->assign('article_print_text', htmlspecialchars($this->language['print'],ENT_COMPAT));
      }

      //SD343: "$is_excerpt_mode_active": no comments if excerpt mode is in effect
      if(!$is_excerpt_mode_active && ($article_arr['settings'] & self::$articlebitfield['displaycomments']))
      {
        $Comments->isArticle = true; //SD341
        $Comments->plugin_id = $this->pluginid;
        $Comments->object_id = $article_arr['articleid'];
        $comments_count      = $Comments->GetCommentsCount();
        $Comments->isArticle = false; //SD341
        $this->_smarty->assign('article_display_comments', true);
        $this->_smarty->assign('article_comment_count', $comments_count);
      }
    }

    //SD341: display admin edit shortcut
    if(!empty($this->settings['admin_edit_link']) && $userinfo['loggedin'] &&
       (
         $userinfo['adminaccess'] ||
         ( (!empty($userinfo['pluginadminids']) && @in_array($this->pluginid, $userinfo['pluginadminids'])) ||
           (!empty($userinfo['admin_pages']) && in_array('articles', $userinfo['admin_pages']) &&
            !empty($userinfo['pluginsubmitids']) && @in_array($this->pluginid, $userinfo['pluginsubmitids']))
         )
       ))
    {
      echo '<a target="_blank" href="'.ROOT_PATH.ADMIN_PATH.'/articles.php'.
           ($this->pluginid>2?'?pluginid='.$this->pluginid.'&amp;':'?').
           'articleaction=displayarticleform&amp;articleid='. $article_arr['articleid'].
           ($mainsettings['enablewysiwyg'] ? '&amp;load_wysiwyg=1' : '').
           '">'.$sdlanguage['common_edit_article'].'</a><br />';

    }
    echo '<a id="p'.$this->pluginid.'_'.$article_arr['articleid'].'"></a>';

    // BIND AND DISPLAY TEMPLATE NOW
    // Check if custom version exists, otherwise use default template:
    $ds = DIRECTORY_SEPARATOR;
    $tmpl_done = false;
    if($this->_isTagPage)
    {
      $tmpl_done = SD_Smarty::display($this->pluginid, 'articles_tagged.tpl'); //SD344
    }

    if(!$tmpl_done && !$this->_isTagPage && !empty($viewing_all_articles))
    {
      if(defined('SD_SMARTY_VERSION') && (SD_SMARTY_VERSION > 2)) //SD344
      {
        $tpls = array();
        if(!empty($this->settings['article_list_template']))
        {
          $tpls[] = $this->settings['article_list_template'];
        }
        $tpls[] = 'articles_list.tpl';
        $tmpl_done = SD_Smarty::display($this->pluginid, $tpls);
      }
      else
      {
        SD_Smarty::setTemplateDir(SD_INCLUDE_PATH.'tmpl'.$ds);
        if(!empty($this->settings['article_list_template']) &&
           is_file(SD_INCLUDE_PATH.'tmpl'.$ds.$this->settings['article_list_template'])) //SD343
        {
          $this->_smarty->display($this->settings['article_list_template']);
          $tmpl_done = true;
        }
        else
        {
          SD_Smarty::setTemplateDir(SD_INCLUDE_PATH.'tmpl'.$ds.'defaults'.$ds);
          if(!empty($this->settings['article_list_template']) &&
             is_file(SD_INCLUDE_PATH.'tmpl'.$ds.'defaults'.$ds.$this->settings['article_list_template'])) //SD343
          {
            $this->_smarty->display($this->settings['article_list_template']);
            $tmpl_done = true;
          }
          else
          if(is_file(SD_INCLUDE_PATH.'tmpl'.$ds.'defaults'.$ds.'articles_list.tpl')) //SD343
          {
            $this->_smarty->display('articles_list.tpl');
            $tmpl_done = true;
          }
        }
      }
    }

    if(!$tmpl_done && empty($viewing_all_articles))
    {
      //SD344: enhanced template handling
      $tpls = array();
      if(!empty($article_arr['template']))
      {
        $tpls[] = $article_arr['template'];
      }
      $tpls[] = 'articles.tpl';
      $tmpl_done = SD_Smarty::display($this->pluginid, $tpls);
    }
    if($tmpl_done) $this->_articleCounter++; //SD344

    // Message for Admin if template wasn't found (displayed only once)
    if(!$tmpl_done && !defined('SD_ARTICLE'.$this->pluginid.'_TMPL_ERROR') && !empty($userinfo['adminaccess']))
    {
      define('SD_ARTICLE'.$this->pluginid.'_TMPL_ERROR',1);
      echo '<p>Articles('.$this->pluginid.') template file <b>NOT FOUND</b>!</p>';
    }

    // display user comments?
    if(!$viewing_all_articles AND ($article_arr['settings'] & self::$articlebitfield['displaycomments']))
    {
      $Comments->isArticle = true; //SD341
      $Comments->plugin_id = $this->pluginid;
      $Comments->object_id = $article_arr['articleid'];
      $Comments->url       = $article_link;
      $Comments->max_comments = (int)$article_arr['max_comments'];

      $allow_comments = ((!empty($userinfo['adminaccess']) || !empty($userinfo['commentaccess'])) || //SD342
                         (!empty($userinfo['plugincommentids']) && in_array($this->pluginid,$userinfo['plugincommentids']))) &&
                         (empty($article_arr['max_comments']) || ($Comments->GetCommentsCount(false) < $article_arr['max_comments']));
      $Comments->allow_comments = $allow_comments;
      $Comments->DisplayComments();
      $Comments->isArticle = false; //SD341
    }

  } //DisplayArticle


  // ############################################################################
  // DISPLAY ARTICLES
  // ############################################################################

  public function DisplayArticles()
  {
    global $DB, $categoryid, $sd_tagspage_link, $sdlanguage, $userinfo;

    // get category settings, if none then resort to default
    $DB->result_type = MYSQL_ASSOC;
    if($categorysettings = $DB->query_first('SELECT * FROM {'.$this->p_prefix.'settings} WHERE categoryid = %d', $categoryid))
    {
      $items_per_page = (int)$categorysettings['maxarticles'];
      $items_per_page = empty($items_per_page) || !is_numeric($items_per_page) ? 10 : (int)$items_per_page;
      $order          = $categorysettings['sorting'];
      $multiplepages  = !empty($categorysettings['multiplepages']);
    }
    else
    {
      $items_per_page = 10;
      $order          = 'newest_first';
      $multiplepages  = 1;
    }

    // Translate sorting
    $default_order = 'IF(IFNULL(datecreated,IFNULL(datestart,0))=0,dateupdated,IFNULL(datecreated,IFNULL(datestart,0))) ';
    if($order == 'oldest_first')
      $order = $default_order.'ASC';
    else
    if(!$order || ($order == 'newest_first'))
      $order = $default_order.'DESC';

    //SD343: display tag cloud
    $article_tags = false;
    if(!empty($this->settings['display_tagcloud']) && ($this->settings['display_tagcloud'] > 0))
    {
      SD_Tags::$maxentries   = 30;
      SD_Tags::$plugins      = $this->pluginid;
      SD_Tags::$tags_title   = $sdlanguage['tags_title'];
      SD_Tags::$targetpageid = 0; // auto-detect (tags page)
      SD_Tags::$tag_as_param = ''; // must be empty
      if($article_tags = SD_Tags::DisplayCloud(false))
      {
        if($this->settings['display_tagcloud'] == 1)
        {
          echo $article_tags;
        }
      }
    }

    $extraquery = '';
    //SD343: check for special params
    // a) first check for tags determined by SD core
    // b) check for params determined by header file
    $this->_tag_active = false;
    global $sd_tag_value;
    $this->_isTagPage = false;
    $TagParams = '';
    $url = RewriteLink('index.php?categoryid=' . $categoryid);

    //SD350: check for search term coming from search engine plugin
    // (is set within header.php)
    if(!empty($this->doSearch) && !empty($this->SearchTerm))
    {
      //TODO: output results here if no Search Engine plugin on same page??
    }
    else
    if(SD_TAG_DETECTED && is_array($sd_tag_value))
    {
      if(!empty($sd_tag_value['key']) && ($sd_tag_value['key']=='tags') && !empty($sd_tag_value['value']))
      {
        $this->_tag_active = $sd_tag_value['value'];
        $extraquery = ' AND EXISTS(SELECT 1 FROM '.PRGM_TABLE_PREFIX.'tags tags WHERE tags.pluginid = '.
                      $this->pluginid.' AND tags.objectid = '.PRGM_TABLE_PREFIX.$this->table_name.
                      ".articleid AND tags.tagtype = 0".
                      " AND lower(tags.tag) = '".
                      ($DB->escape_string(strtolower($this->_tag_active)))."')\r\n";
        $this->_isTagPage = true;
        $TagParams = $this->_tag_active;
        $url = $sd_tagspage_link.$TagParams;
      }
      else
      if(!empty($sd_tag_value['year']) && !empty($sd_tag_value['month']) &&
         Is_Valid_Number($sd_tag_value['year'],0,2000,2050) &&
         Is_Valid_Number($sd_tag_value['month'],0,1,12) )
      {
        $this->_tag_active = '';
        $extraquery = " AND DATE_FORMAT(FROM_UNIXTIME(IF(datestart>0,datestart,datecreated)),'%Y%m') = ".
                      sprintf('%02d',$sd_tag_value['year']).sprintf('%02d',$sd_tag_value['month']);
        $this->_isTagPage = true;
        $TagParams = sprintf('%02d',$sd_tag_value['year']).'/'.sprintf('%02d',$sd_tag_value['month']);
        $url = $sd_tagspage_link.$TagParams;
      }
      else
      {
        // Invalid search/tag specified! Create empty results!
        $extraquery = ' AND 1 = 0 ';
      }
    }
    else
    if(!empty($this->slug_arr['id']))
    {
      if(!empty($this->slug_arr['key']) && ($this->slug_arr['key']=='tags') && !empty($this->slug_arr['value']))
      {
        $this->_tag_active = $this->slug_arr['value'];
        $extraquery = ' AND EXISTS(SELECT 1 FROM '.PRGM_TABLE_PREFIX.'tags tags WHERE tags.pluginid = '.
                      $this->pluginid.' AND tags.objectid = '.PRGM_TABLE_PREFIX.$this->table_name.
                      ".articleid AND tags.tagtype = 0".
                      " AND lower(tags.tag) = '".
                      ($DB->escape_string(strtolower($this->_tag_active)))."')\r\n";
        $this->_isTagPage = true;
        $TagParams = $this->_tag_active;
      }
      else
      if(!empty($this->slug_arr['year']) && !empty($this->slug_arr['month']) &&
         Is_Valid_Number($this->slug_arr['year'],0,2000,2050) &&
         Is_Valid_Number($this->slug_arr['month'],0,1,12) )
      {
        $this->_tag_active = '';
        $extraquery = " AND DATE_FORMAT(FROM_UNIXTIME(IF(datestart>0,datestart,datecreated)),'%Y%m') = ".
                      sprintf('%02d',$this->slug_arr['year']).sprintf('%02d',$this->slug_arr['month']);
        $this->_isTagPage = true;
        $TagParams = sprintf('%02d',$this->slug_arr['year']).'/'.sprintf('%02d',$this->slug_arr['month']);
        $url = preg_replace('#'.$this->_quoted_url_ext.'$#', '/',
                            RewriteLink('index.php?categoryid='.$categoryid)).$TagParams;
      }
      else
      {
        // Invalid search/tag specified! Create empty results!
        $extraquery = ' AND 1 = 0 ';
      }
    }

    // if articles are hidden by default, then find the ones with 'display on main page listing' enabled
    if(empty($sd_tag_value['id']))
    {
      if(($this->globalsettings & self::$articlebitfield['displaymainpage']) == 0)
        $extraquery .= ' AND (settings & '.self::$articlebitfield['displaymainpage'].')';
      else
        $extraquery .= ' AND ((settings & 1) OR (settings & '.self::$articlebitfield['displaymainpage'].'))';
    }

    //SD342: check for usergroup permission's
    $extraquery .= " AND ((IFNULL(access_view,'')='') OR (access_view like '%|".(int)$userinfo['usergroupid']."|%'))";

    $where = ' WHERE (settings & '.self::$articlebitfield['displayonline'].')';
    //v3.5.2: added "slug_arr['id']" to ignore categories
    if( (empty($sd_tag_value['id']) && empty($this->slug_arr['id'])) ||
        empty($this->settings['ignore_pages_for_date_tags']) ) //SD360
    {
      $where .= ' AND ((categoryid = '.$categoryid.')'.
                ' OR EXISTS(SELECT 1 FROM {p'.$this->pluginid.'_articles_pages} ap'.
                ' WHERE ap.categoryid = '.$categoryid.
                ' AND ap.articleid = {'.$this->table_name.'}.articleid))';
    }
    $where .= ' AND ((datestart = 0) OR (datestart < ' . TIME_NOW . '))';

    if(empty($this->settings['ignore_publish_end_date'])) //news plugin v3.5.2 (SD36)
    {
      $where .= ' AND ((dateend = 0) OR (dateend > ' . TIME_NOW . '))';
    }

    $total_rows = $DB->query_first('SELECT COUNT(*) article_count'.
                                   ' FROM {'.$this->table_name.'}'.
                                   $where.$extraquery);

    if(!$total_rows = (int)$total_rows['article_count'])
    {
      global $pages_md_arr;
      //SD343: display "no articles" message
      echo str_replace(array('[pagename]','[pagetitle]','[pluginid]','[pageurl]','[siteurl]'),
                       array($pages_md_arr[$categoryid]['name'],
                             $pages_md_arr[$categoryid]['title'],
                             $this->pluginid,
                             RewriteLink(),
                             SITE_URL),
                       $this->settings['no_results_message']);
      return false;
    }

    // Check and validate page and pagesize params
    $page = Is_Valid_Number(GetVar($this->p_prefix.'start', 1, 'whole_number'),1,1,999999999);
    $items_per_page = Is_Valid_Number($items_per_page,1,1,9999);
    if($items_per_page > $total_rows) $items_per_page = $total_rows;
    if($page * $items_per_page > $total_rows) $page = ceil($total_rows / $items_per_page); //SD343: ceil, not floor
    if($page < 1) $page = 1;

    $limit = ' LIMIT '.(($page-1)*$items_per_page).','.$items_per_page;
    $noArticles = false;
    if($get_articles = $DB->query(
       'SELECT * FROM {'.$this->table_name.'}'.
       $where . $extraquery .
       ' ORDER BY (settings & '.
       self::$articlebitfield['displaysticky'].') DESC, '.
       $order.$limit))
    {
      if($article_rows = $DB->get_num_rows($get_articles))
      {
        echo '
        <div id="p'.$this->pluginid.'_container">';

        //SD343: a tag may be used for article selection, display it in container:
        if($this->_isTagPage)
        {
          if(!empty($sd_tag_value['id']))
          {
            echo '
            <div id="p'.$this->pluginid.'_tag_head">';
            if(!empty($this->_tag_active) && ($sd_tag_value['id']='tags'))
              echo str_replace('[tag]', (string)$this->_tag_active, $this->language['article_tags_head']);
            else
            if(!empty($sd_tag_value['month']) && !empty($sd_tag_value['year']))
            {
              global $sd_months_arr;
              echo str_replace(array('[month]','[year]'),
                               array($sd_months_arr[$sd_tag_value['month']],
                                     sprintf('%04d',$sd_tag_value['year'])),
                               $this->language['article_year_month_head']);
            }
            echo '
            </div>';
          }
          else
          if(!empty($this->slug_arr['id']))
          {
            echo '
            <div id="p'.$this->pluginid.'_tag_head">';
            if(!empty($this->_tag_active) && ($this->slug_arr['id']='tags'))
              echo str_replace('[tag]', (string)$this->_tag_active, $this->language['article_tags_head']);
            else
            if(!empty($this->slug_arr['month']) && !empty($this->slug_arr['year']))
            {
              global $sd_months_arr;
              echo str_replace(array('[month]','[year]'),
                               array($sd_months_arr[$this->slug_arr['month']],
                                     sprintf('%04d',$this->slug_arr['year'])),
                               $this->language['article_year_month_head']);
            }
            echo '
            </div>';
          }
        }

        $this->_articleCounter = 0;
        for($article_count = 1; $article_arr = $DB->fetch_array($get_articles,null,MYSQL_ASSOC); $article_count++)
        {
          $this->DisplayArticle($article_arr, true);

          if(!empty($limit) && ($article_count < $article_rows))
          {
            echo $this->settings['article_separator'];
          }
        } //for

        // Articles Pagination
        if($multiplepages && ($total_rows > $items_per_page))
        {
          $p = new pagination;
          $p->enablePrevNext = true; //SD343
          $p->parameterName($this->p_prefix.'start');
          $p->items($total_rows);
          $p->limit($items_per_page);
          $p->currentPage($page);
          $p->adjacents(3);
          $p->target($url);
          $p->show();
        }

        //SD343: display tag cloud
        if(!empty($article_tags) && !empty($this->settings['display_tagcloud']) && ($this->settings['display_tagcloud']==2))
        {
          echo $article_tags;
        }

        echo '
        </div><!-- end #p'.$this->pluginid.'_container -->
        ';
      }
      else
      //SD343: display "tag not found" error:
      if($this->_tag_active !== false)
      {
        echo '
        <div id="p'.$this->pluginid.'_container">
          <div id="p'.$this->pluginid.'_tag_head">
          '.str_replace('[tag]', (string)$this->_tag_active, $this->language['msg_no_tag_results']).'
          </div>
        </div><!-- end #p'.$this->pluginid.'_container -->';
      }
      else
      {
        $noArticles = true;
      }
    }
    else
    {
      $noArticles = true;
    }

    if($noArticles)
    {
      echo '
      <div id="p'.$this->pluginid.'_container">
        <div class="noarticles">'.$this->language['msg_no_results'].'</div>
      </div>';
    }
    $DB->result_type = MYSQL_BOTH;

    unset($this->_attachment, $noArticles);

  } //DisplayArticles


  // ############################################################################
  // SELECT FUNCTION
  // ############################################################################
  public function Display($article_arr)
  {
    global $DB, $categoryid, $userinfo, $mainsettings_search_results_page;

    //SD360: if search term was indicated, do nothing and assume, that the
    // Search Engine plugin takes care of results
    if($this->doSearch && ($categoryid == $mainsettings_search_results_page))
    {
      return true;
    }

    $extra = '';
    if(empty($this->settings['ignore_publish_end_date'])) //news plugin v3.5.2 (SD3.5.1)
    {
      $extra = ' AND ((dateend = 0) OR (dateend > %d))';
    }

    $articleid = GetVar($this->p_prefix.'articleid', false, 'whole_number');
    if(!empty($article_arr) && ($article_arr['pluginid']==$this->pluginid))
    {
      //news plugin v3.5.2 (SD3.5.1): check for dateend if cached
      if(!empty($this->settings['ignore_publish_end_date']) ||
         empty($article_arr['dateend']) ||
         ($article_arr['dateend'] >= TIME_NOW) )
      {
        $this->DisplayArticle($article_arr, false, true);
      }
    }
    else
    if($articleid && ($article = $DB->query_first(
        'SELECT * FROM '.PRGM_TABLE_PREFIX.$this->table_name.' news WHERE articleid = %d'.
        ' AND ((categoryid = %d) OR EXISTS (SELECT 1 FROM '.PRGM_TABLE_PREFIX.'p'.$this->pluginid.'_articles_pages ap'.
        ' WHERE ap.articleid = news.articleid AND ap.categoryid = %d))'.
        ' AND (settings & 2) AND ((datestart = 0) OR (datestart < %d)) '.$extra,
         $articleid, $categoryid, $categoryid, TIME_NOW, TIME_NOW)))
    {
      $this->DisplayArticle($article);
    }
    else
    {
      // display easy "submit article to this page" link
      if( $userinfo['loggedin'] && defined('DISPLAY_PLUGIN_ADMIN_SHORTCUTS') && DISPLAY_PLUGIN_ADMIN_SHORTCUTS &&
          ( $userinfo['adminaccess'] ||
            ( (!empty($userinfo['pluginadminids']) && @in_array($this->pluginid, $userinfo['pluginadminids'])) ||
              (!empty($userinfo['admin_pages']) && in_array('articles', $userinfo['admin_pages']) &&
               !empty($userinfo['pluginsubmitids']) && @in_array($this->pluginid, $userinfo['pluginsubmitids']))
            )
          )
        )
      {
        echo IMAGE_EDIT . ' <a href="'.ADMIN_PATH.'/articles.php'.
             ($this->pluginid>2?'?pluginid='.$this->pluginid:'').
             '&amp;articleaction=displayarticleform&amp;load_wysiwyg=1&amp;pageid='.$categoryid.'">'.
             $this->language['insert_article_into_page'] . '</a><br /><br />';
      }

      $this->DisplayArticles();
    }

  } //Display

} // end of class
} // DO NOT REMOVE!
